# -*- coding: utf-8 -*-
"""
    pip_services_container.refer.BuildReferencesDecorator
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Build references decorator implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services_commons.refer import IReferences
from pip_services_commons.refer import Descriptor
from pip_services_commons.refer import ReferenceException
from pip_services_commons.build import IFactory

from .ReferencesDecorator import ReferencesDecorator

class BuildReferencesDecorator(ReferencesDecorator):
    build_enabled = True

    def __init__(self, base_references, parent_references):
        super(BuildReferencesDecorator, self).__init__(base_references, parent_references)


    def find_factory(self, locator):
        components = self.get_all()
        for component in components:
            if isinstance(component, IFactory):
                if component.can_create(locator) != None:
                    return component
        return None


    def create(self, locator, factory):
        if factory == None:
            return None

        try:
            # Create component
            return factory.create(locator)
        except Exception as ex:
            return None


    def clarify_locator(self, locator, factory):
        if factory == None:
            return locator
        if not isinstance(locator, Descriptor):
            return locator
        
        another_locator = factory.can_create(locator)
        if another_locator == None:
            return locator
        if not isinstance(another_locator, Descriptor):
            return locator
        
        descriptor = locator
        another_descriptor = another_locator
        
        return Descriptor(
            descriptor.get_group() if descriptor.get_group() != None else another_descriptor.get_group(),
            descriptor.get_type() if descriptor.get_type() != None else another_descriptor.get_type(),
            descriptor.get_kind() if descriptor.get_kind() != None else another_descriptor.get_kind(),
            descriptor.get_name() if descriptor.get_name() != None else another_descriptor.get_name(),
            descriptor.get_version() if descriptor.get_version() != None else another_descriptor.get_version()
        )


    def find(self, query, required):
        components = super(BuildReferencesDecorator, self).find(query, False)
        locator = query.locator

        # Try to create component
        if len(components) == 0 and self.build_enabled:
            factory = self.find_factory(locator)
            component = self.create(locator, factory)
            if component != None:
                try:
                    locator = self.clarify_locator(locator, factory)
                    self.parent_references.put(locator, component)
                    components.append(component)
                except Exception as ex:
                    # Ignore exception
                    pass

        # Throw exception is no required components found
        if required and len(components) == 0:
            raise ReferenceException(null, locator)

        return components



# -*- coding: utf-8 -*-
"""
    pip_services_container.config.ContainerConfigReader
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container configuration reader implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services_commons.errors import ConfigException
from pip_services_commons.config import JsonConfigReader
from pip_services_commons.config import YamlConfigReader
from .ContainerConfig import ContainerConfig

class ContainerConfigReader(object):

    @staticmethod
    def read_from_file(correlation_id, path):
        if path == None:
            raise ConfigException(correlation_id, "NO_PATH", "Missing config file path")
        
        index = path.rfind('.')
        ext = path[index + 1:].lower() if index > 0 else ''

        if ext == "json":
            return ContainerConfigReader.read_from_json_file(correlation_id, path)
        elif ext == "yaml":
            return ContainerConfigReader.read_from_yaml_file(correlation_id, path)
        
        # By default read as JSON
        return ContainerConfigReader.read_from_json_file(correlation_id, path)

    @staticmethod
    def read_from_json_file(correlation_id, path):
        config = JsonConfigReader.read_config(correlation_id, path)
        return ContainerConfig.from_config(config)

    @staticmethod
    def read_from_yaml_file(correlation_id, path):
        config = YamlConfigReader.read_config(correlation_id, path)
        return ContainerConfig.from_config(config)




# -*- coding: utf-8 -*-
"""
    pip_services_container.refer.ManagedReferences
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Managed references implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services_commons.refer import References
from pip_services_commons.refer import Referencer
from pip_services_commons.run import IOpenable
from pip_services_commons.run import Opener
from pip_services_commons.run import Closer

from .ReferencesDecorator import ReferencesDecorator
from .BuildReferencesDecorator import BuildReferencesDecorator
from .LinkReferencesDecorator import LinkReferencesDecorator
from .RunReferencesDecorator import RunReferencesDecorator

class ManagedReferences(ReferencesDecorator, IOpenable):
    _references = None
    _builder = BuildReferencesDecorator
    _linker = LinkReferencesDecorator
    _runner = RunReferencesDecorator

    def __init__(self, tuples = None):
        super(ManagedReferences, self).__init__(None, None)

        self._references = References(tuples)
        self._builder = BuildReferencesDecorator(self._references, self)
        self._linker = LinkReferencesDecorator(self._builder, self)
        self._runner = RunReferencesDecorator(self._linker, self)

        self.base_references = self._runner

    def is_opened(self):
        components = self.get_all()
        return Opener.is_opened(components)

    def open(self, correlation_id):
        components = self.get_all()
        Referencer.set_references(self, components)
        Opener.open(correlation_id, components)

    def close(self, correlation_id):
        components = self.get_all()
        Closer.close(correlation_id, components)
        Referencer.unset_references(components)

    @staticmethod
    def from_tuples(*tuples):
        return ManagedReferences(tuples)


# -*- coding: utf-8 -*-
"""
    pip_services_container.ProcessContainer
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Process container implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

import sys
import signal
import time
import threading

from pip_services_commons.log import ConsoleLogger
from .Container import Container

class ProcessContainer(Container):
    _exit_event = None

    def __init__(self):
        super(ProcessContainer, self).__init__()
        self._logger = ConsoleLogger()
        self._exit_event = threading.Event()

    def read_config_from_args_or_file(self, correlation_id, default_path):
        args = sys.argv
        config_path = args[1] if len(args) > 1 else default_path
        self.read_config_from_file(correlation_id, config_path)

    def _capture_errors(self, correlation_id):
        def handle_exception(exc_type, exc_value, exc_traceback):
            self._logger.fatal(correlation_id, exc_value, "Process is terminated")
            self._exit_event.set()
            #sys.exit(1)

        sys.excepthook = handle_exception

    def _capture_exit(self, correlation_id):
        self._logger.info(correlation_id, "Press Control-C to stop the microservice...")

        def sigint_handler(signum, frame):
            self._logger.info(correlation_id, "Goodbye!")
            self._exit_event.set()
            #sys.exit(1)
            
        signal.signal(signal.SIGINT, sigint_handler)
        signal.signal(signal.SIGTERM, sigint_handler)

        # Wait and close
        self._exit_event.clear()
        while not self._exit_event.is_set():
            try:
                self._exit_event.wait(1)
            except:
                pass # Do nothing...

    def run(self, correlation_id):
        self._capture_errors(correlation_id)
        self.start(correlation_id)
        self._capture_exit(correlation_id)
        self.stop(correlation_id)

    def run_with_config(self, correlation_id, config):
        self.set_config(config)
        self.run(correlation_id)

    def run_with_config_file(self, correlation_id, path):
        self.read_config_from_file(correlation_id, path)
        self.run(correlation_id)

    def run_with_config_from_args_or_file(self, correlation_id, default_path):
        self.read_config_from_args_or_file(correlation_id, default_path)
        self.run(correlation_id)



# -*- coding: utf-8 -*-
"""
    pip_services_container.build.DefaultContainerFactory
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Default container factory implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services_commons.refer import Descriptor
from pip_services_commons.build import CompositeFactory
from pip_services_commons.log import DefaultLoggerFactory
from pip_services_commons.count import DefaultCountersFactory
from pip_services_commons.config import DefaultConfigReaderFactory
from pip_services_commons.cache import DefaultCacheFactory
from pip_services_commons.auth import DefaultCredentialStoreFactory
from pip_services_commons.connect import DefaultDiscoveryFactory
from ..info.ContainerInfoFactory import ContainerInfoFactory

DefaultContainerFactoryDescriptor = Descriptor(
    "pip-services-container", "factory", "container", "default", "1.0"
)

class DefaultContainerFactory(CompositeFactory):

    def __init__(self):
        super(DefaultContainerFactory, self).__init__()
        self.add(ContainerInfoFactory())
        self.add(DefaultLoggerFactory())
        self.add(DefaultCountersFactory())
        self.add(DefaultConfigReaderFactory())
        self.add(DefaultCacheFactory())
        self.add(DefaultCredentialStoreFactory())
        self.add(DefaultDiscoveryFactory())




# -*- coding: utf-8 -*-
"""
    pip_services_container.info.ContainerInfoFactory
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container info factory implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from .ContainerInfo import ContainerInfo

from pip_services_commons.refer import Descriptor
from pip_services_commons.build import Factory

ContainerInfoFactoryDescriptor = Descriptor(
    "pip-services-container", "factory", "container-info", "default", "1.0"
)

ContainerInfoDescriptor = Descriptor(
    "pip-services-container", "container-info", "default", "default", "1.0"
)

class ContainerInfoFactory(Factory):

    def __init__(self):
        self.register_as_type(ContainerInfoDescriptor, ContainerInfo)



# -*- coding: utf-8 -*-
"""
    pip_services_container.refer.ReferencesDecorator
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    References decorator implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services_commons.refer import IReferences
from pip_services_commons.refer import ReferenceQuery

class ReferencesDecorator(object, IReferences):
    base_references = None
    parent_references = None

    def __init__(self, base_references, parent_references):
        self.base_references = base_references if base_references != None else parent_references
        self.parent_references = parent_references if parent_references != None else base_references


    def put(self, locator, component):
        self.base_references.put(locator, component)

    def remove(self, locator):
        return self.base_references.remove(locator)

    def remove_all(self, locator):
        return self.base_references.remove_all(locator)

    def get_all(self):
        return self.base_references.get_all()
        
    def get_one_optional(self, locator):
        try:
            components = self.find(ReferenceQuery(locator), False)
            return components[0] if len(components) > 0 else None
        except Exception as ex:
            return None

    def get_one_required(self, locator):
        components = self.find(ReferenceQuery(locator), True)
        return components[0] if len(components) > 0 else None

    def get_optional(self, locator):
        try:
            return self.find(ReferenceQuery(locator), False)
        except Exception as ex:
            return []

    def get_required(self, locator):
        return self.find(ReferenceQuery(locator), True)

    def find(self, query, required):
        return self.base_references.find(query, required)



# -*- coding: utf-8 -*-
"""
    pip_services_container.build.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container build module initialization
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = ['DefaultContainerFactory']

from .DefaultContainerFactory import DefaultContainerFactory



# -*- coding: utf-8 -*-
"""
    pip_services_container.refer.LinkReferencesDecorator
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Link references decorator implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services_commons.refer import IReferences
from pip_services_commons.refer import Referencer

from .ReferencesDecorator import ReferencesDecorator

class LinkReferencesDecorator(ReferencesDecorator):
    link_enabled = True

    def __init__(self, base_references, parent_references):
        super(LinkReferencesDecorator, self).__init__(base_references, parent_references)


    def put(self, locator, component):
        super(LinkReferencesDecorator, self).put(locator, component)

        if self.link_enabled:
            Referencer.set_references_for_one(self.parent_references, component)


    def remove(self, locator):
        component = super(LinkReferencesDecorator, self).remove(locator)

        if self.link_enabled:
            Referencer.unset_references_for_one(component)

        return component


    def remove_all(self, locator):
        components = super(LinkReferencesDecorator, self).remove_all(locator)

        if self.link_enabled:
            Referencer.unset_references(components)

        return components



# -*- coding: utf-8 -*-
"""
    pip_services_container.refer.RunReferencesDecorator
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Run references decorator implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services_commons.refer import IReferences
from pip_services_commons.refer import ReferenceQuery
from pip_services_commons.run import Opener
from pip_services_commons.run import Closer

from .ReferencesDecorator import ReferencesDecorator

class RunReferencesDecorator(ReferencesDecorator):
    open_enabled = True
    close_enabled = True

    def __init__(self, base_references, parent_references):
        super(RunReferencesDecorator, self).__init__(base_references, parent_references)


    def put(self, locator, component):
        super(RunReferencesDecorator, self).put(locator, component)

        if self.open_enabled:
            Opener.open_one(None, component)


    def remove(self, locator):
        component = super(RunReferencesDecorator, self).remove(locator)

        if self.close_enabled:
            Closer.close_one(None, component)

        return component


    def remove_all(self, locator):
        components = super(RunReferencesDecorator, self).remove_all(locator)

        if self.close_enabled:
            Closer.close(None, components)

        return components



# -*- coding: utf-8 -*-
"""
    pip_services_container.config.ComponentConfig
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Component configuration implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services_commons.reflect import TypeDescriptor
from pip_services_commons.refer import Descriptor
from pip_services_commons.errors import ConfigException

class ComponentConfig(object):
    descriptor = None
    #type = None
    config = None

    def __init__(self, descriptor = None, tipe = None, config = None):
        self.descriptor = descriptor
        self.type = tipe
        self.config = config

    @staticmethod
    def from_config(config):
        descriptor = Descriptor.from_string(config.get_as_nullable_string("descriptor"))
        tipe = TypeDescriptor.from_string(config.get_as_nullable_string("type")) 
        
        if descriptor == None and tipe == None:
            raise ConfigException(None, "BAD_CONFIG", "Component configuration must have descriptor or type")
        
        return ComponentConfig(descriptor, tipe, config)



# -*- coding: utf-8 -*-
"""
    pip_services_container.config.ContainerConfig
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container configuration implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

from pip_services_commons.config import ConfigParams
from .ComponentConfig import ComponentConfig

class ContainerConfig(list):

    def __init__(self, components = None):
        if components != None:
            for component in components:
                self.append(component)

    @staticmethod
    def from_value(value):
        config = ConfigParams.from_value(value)
        return from_config(config)

    @staticmethod
    def from_config(config):
        result = ContainerConfig()
        if config == None:
            return result
        
        for section in config.get_section_names():
            component_config = config.get_section(section)
            result.append(ComponentConfig.from_config(component_config))
        
        return result



# -*- coding: utf-8 -*-
"""
    pip_services_container.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container module initialization
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = [ 'Container', 'ProcessContainer' ]

from .Container import Container
from .ProcessContainer import ProcessContainer




# -*- coding: utf-8 -*-
"""
    pip_services_container.Container
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

import traceback

from pip_services_commons.log import NullLogger
from pip_services_commons.log import CompositeLogger
from pip_services_commons.errors import InvalidStateException
from pip_services_commons.refer import Descriptor
from pip_services_commons.run import Opener
from pip_services_commons.run import Closer
from pip_services_commons.refer import Referencer

from .build.DefaultContainerFactory import DefaultContainerFactory
from .build.DefaultContainerFactory import DefaultContainerFactoryDescriptor
from .info.ContainerInfo import ContainerInfo
from .config.ContainerConfigReader import ContainerConfigReader
from .refer.ContainerReferences import ContainerReferences

class Container(object):
    _logger = None
    _info = None
    _config = None
    _references = None

    def __init__(self, config = None):
        self._logger = NullLogger()
        self._info = ContainerInfo()
        self._references = ContainerReferences()
        self._config = config

    def get_info(self):
        return self._info

    def set_info(self, info):
        self._info = info

    def get_config(self):
        return self._config

    def set_config(self, value):
        self._config = value

    def get_references(self):
        return self._references

    def set_references(self, references):
        self._references = references

    def read_config_from_file(self, correlation_id, path):
        self._config = ContainerConfigReader.read_from_file(correlation_id, path)
        
    def _init_references(self, references):
        # Override in base classes
        references.put(DefaultContainerFactoryDescriptor, DefaultContainerFactory())

    def start(self, correlation_id):
        if self._references == None:
            raise InvalidStateException(correlation_id, "NO_CONFIG", "Container was not configured")
                
        try:
            self._logger.trace(correlation_id, "Starting container.")
            
            # Create references with configured components
            self._init_references(self._references)
            self._references.put_from_config(self._config)

            # Reference and open components
            components = self._references.get_all()
            Referencer.set_references(self._references, components)
            Opener.open(correlation_id, components)

            # Get reference to logger
            self._logger = CompositeLogger(self._references)
            
            # Get reference to container info
            info_descriptor = Descriptor("*", "container-info", "*", "*", "*")
            self._info = self._references.get_one_required(info_descriptor)
            self._logger.info(correlation_id, "Container " + self._info.name + " started.")
        except Exception as ex:
            self._references = None
            self._logger.error(correlation_id, ex, "Failed to start container")
            traceback.print_exc()
            raise ex

    def stop(self, correlation_id):
        if self._references == None:
            raise InvalidStateException(correlation_id, "NO_STARTED", "Container was not started")
                
        try:
            self._logger.trace(correlation_id, "Stopping " + self._info.name + " container")

            # Close and deference components
            self._references.close(correlation_id)

            self._logger.info(correlation_id, "Container " + self._info.name + " stopped")
        except Exception as ex:
            self._logger.error(correlation_id, ex, "Failed to stop container")
            raise ex



# -*- coding: utf-8 -*-
"""
    pip_services_container.info.ContainerInfo
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container info implementation
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

import datetime

from pip_services_commons.data import StringValueMap
from pip_services_commons.data import IdGenerator
from pip_services_commons.config import ConfigParams
from pip_services_commons.config import IReconfigurable

class ContainerInfo(object, IReconfigurable):
    name = None
    description = None
    container_id = None
    start_time = None
    properties = None

    def __init__(self):
        self.name = "unknown"
        self.start_time = datetime.datetime.utcnow()
        self.container_id = IdGenerator.next_long()


    def configure(self, config):
        self.name = config.get_as_string_with_default("name", self.name)
        self.name = config.get_as_string_with_default("info.name", self.name)

        self.description = config.get_as_string_with_default("description", self.description)
        self.description = config.get_as_string_with_default("info.description", self.description)

        self.properties = config.get_section("properties")


    def uptime():
        current_time = datetime.datetime.utcnow()
        return current_time - start_time


    @staticmethod
    def from_config(config):
        result = ContainerInfo()
        result.configure(config)
        return result




# -*- coding: utf-8 -*-
"""
    pip_services_container.config.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container config module initialization
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = [
    'ComponentConfig', 'ContainerConfig', 'ContainerConfigReader'
]

from .ComponentConfig import ComponentConfig
from .ContainerConfig import ContainerConfig
from .ContainerConfigReader import ContainerConfigReader



# -*- coding: utf-8 -*-
"""
    pip_services_container.refer.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container refer module initialization
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = [ 'ReferencesDecorator', 'RunReferencesDecorator', 'LinkReferencesDecorator',
    'BuildReferencesDecorator', 'ManagedReferences', 'ContainerReferences' ]

from .ReferencesDecorator import ReferencesDecorator
from .RunReferencesDecorator import RunReferencesDecorator
from .LinkReferencesDecorator import LinkReferencesDecorator
from .BuildReferencesDecorator import BuildReferencesDecorator
from .ManagedReferences import ManagedReferences
from .ContainerReferences import ContainerReferences


# -*- coding: utf-8 -*-
"""
    pip_services_container.info.__init__
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Container info module initialization
    
    :copyright: Conceptual Vision Consulting LLC 2015-2016, see AUTHORS for more details.
    :license: MIT, see LICENSE for more details.
"""

__all__ = [ 'ContainerInfo', 'ContainerInfoFactory' ]

from .ContainerInfo import ContainerInfo
from .ContainerInfoFactory import ContainerInfoFactory





"""
Pip.Services Container
----------------------

Pip.Services is an open-source library of basic microservices.
pip_services_container provides IoC container implementation.

Links
`````

* `website <http://www.pipservices.org>`_
* `development version <http://github.com/pip-services/pip-services-container-python>`

"""

from setuptools import setup
from setuptools import find_packages

setup(
    name='pip_services_container',
    version='2.0.0',
    url='http://github.com/pip-services/pip-services-container-python',
    license='MIT',
    author='Conceptual Vision Consulting LLC',
    description='IoC container for Pip.Services in Python',
    long_description=__doc__,
    packages=find_packages(exclude=['config', 'data', 'examples', 'test']),
    include_package_data=True,
    zip_safe=True,
    platforms='any',
    install_requires=[
        'iso8601', 'PyYAML', 'pip_services_commons'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]    
)
